/* PerforceVcsProviderType.java $Revision:$  $Date:$
 *
 * Copyright (C) 2001  Mike Swainston-Rainford
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; eitherd
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.togethersoft.modules.p4vcs;

import com.perforce.api.Client;
import com.perforce.api.Debug;
import com.perforce.api.Env;
import com.perforce.api.EventLog;
import com.togethersoft.modules.p4vcs.dialogs.SelectPasswordDialog;
import com.togethersoft.openapi.ide.IdeAccess;
import com.togethersoft.openapi.ide.config.IdeConfig;
import com.togethersoft.openapi.ide.config.IdeConfigEditor;
import com.togethersoft.openapi.ide.config.IdeConfigLevel;
import com.togethersoft.openapi.ide.config.IdeConfigManagerAccess;
import com.togethersoft.openapi.ide.message.IdeMessageManagerAccess;
import com.togethersoft.openapi.ide.message.IdeMessagePage;
import com.togethersoft.openapi.ide.message.IdeMessageType;
import com.togethersoft.openapi.vcs.VcsProvider;
import com.togethersoft.openapi.vcs.VcsProviderConstants;
import com.togethersoft.openapi.vcs.VcsProviderException;
import com.togethersoft.openapi.vcs.VcsProviderHelper;
import com.togethersoft.openapi.vcs.VcsProviderRegistryAccess;
import com.togethersoft.openapi.vcs.VcsProviderType;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Properties;

/**
 * This class defines a Version Control System provider for
 * <code>PerforceVcsProvider</code> instances. <code>PerforceVcsProviderType</code> and <code>PerforceVcsProvider</code> are
 * identified with the ID "P4".
 * @author <a href="mailto:mike@mjs-r.com">Mike Swainston-Rainford</a>
 * @version $Date:$ $Revision:$
 */
public class PerforceVcsProviderType implements VcsProviderType, VcsProviderConstants, PropertyChangeListener
{
   /**
    * Creates new instance of <code>PerforceVcsProvider</code>.
    * @return <code>VcsProvider</code> interface
    */
   public VcsProvider newVcsProvider() throws VcsProviderException
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: newVcsProvider called");
      }
      PerforceVcsProvider pvp = new PerforceVcsProvider();
      VcsProviderHelper.getInstance().registerProvider(pvp);
      return pvp;
   }

   /**
    * Returns description of the provider.
    * @return <code>String</code> "Perforce P4 Provider"
    */
   public String getProviderName()
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: getProviderName called");
      }
      return "Perforce P4 Provider";
   }

   /**
    * Returns a string which is a unique identifier of <code>VcsProvider</code>.
    * @return <code>String</code> "P4"
    */
   public String getID()
   {
      //    if (debugOn && messagepage != null) {
      //      messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: getID called");
      //    }
      return "P4";
   }

   /**
    * Performs initialization of the Perforce vcs. Loads the Perforce <code>Properties</code> and creates the
    * <code>Env</code> object from p4package. Sets the p4package <code>Debug</code> levels.
    * @throw <code>VcsProviderException</code> if fatal error occurs
    */
   public void initialize() throws VcsProviderException
   {
      if (isInitialized())
         throw new VcsProviderException("P4 provider already initialized");
      IdeConfigEditor editor = IdeConfigManagerAccess.getConfigManager().getConfigEditor(IdeConfigLevel.PROJECT,
          "Version Control", true);
      if (editor != null)
      {
         editor.addPropertyChangeListener(this);
      }
      messagepage = IdeMessageManagerAccess.getMessageManager().openPage("Version Control");
      if (messagepage != null)
      {
         messagepage.setTitle("Version Control");
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: Provider initialized");
      }
      myProperties = loadProperties();
      try
      {
         myHost = InetAddress.getLocalHost().getHostName();
      }
      catch (UnknownHostException uhe)
      {
         myHost = "";
      }

      if (null == myProperties.getProperty("p4.password") || "".equals(myProperties.getProperty("p4.password")))
      {
         String user = myProperties.getProperty("p4.user");
         SelectPasswordDialog selectPasswordDialog = new SelectPasswordDialog(user);
         selectPasswordDialog.showDialog();
         if (selectPasswordDialog.isOK())
         {
            myProperties.setProperty("p4.user", selectPasswordDialog.getUser());
            myProperties.setProperty("p4.password", selectPasswordDialog.getPassword());
            //To Do test that password is OK
         }
         else
         {
           return;  
         }
      }      
      
      p4Env = new Env(getProperties());
      Debug.setProperties(getProperties());
      String logFileName = getProperties().getProperty("p4.logfile");
      EventLog eventLog = null;
      if (null != logFileName && !"".equals(logFileName))
      {
         eventLog = new EventLog("Together Perforce Log", logFileName);
      } else
      {
         eventLog = new EventLog("Together Perforce Log");
      }
      Debug.setEventLog(eventLog);
       
      Client.loadClients(getEnv());
      isInitialised = true;
   }

   /** Uninitilizes the p4package and unregisters all Perforce provider instances. */
   public void unInitialize()
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: unInitialize called");
      }
      isInitialised = false;
   }

   /**
    * Returns <code>true</code> if the Perforce provider is initialized.
    * @return <code>true</code> if initialized, <code>false</code> otherwise
    */
   public boolean isInitialized()
   {
      //    if (debugOn && messagepage != null) {
      //      messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: isInitialized called");
      //    }
      return isInitialised;
   }

   /**
    * To Do: make sure all these commands work at project level - else remove from check
    * Returns <code>true</code> if <CODE>command</CODE> is supported by the Perforce provider for project operations.
    * @return <code>true</code> if <CODE>command</CODE> is supported , <code>false</code> otherwise
    */
   public boolean isCommandSupportedForProject(String command)
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: isCommandSupportedForProject called");
      }
      return command.equals(CMD_GET_FILES_STATUSES) || command.equals(CMD_UPDATE_FILES) || command.equals(CMD_GET_FILES) ||
          command.equals(CMD_ADD_FILES);
   }

   /**
    * Returns <code>true</code> if the Perforce provider supports advanced options for <CODE>command</CODE>,
    * that is a call to <CODE>PerforceVcsProvider.showAdvancedOptions</CODE> with this command as
    * parameter will not be ignored.
    * @return <code>true</code> if <CODE>command</CODE> supports advanced options,
    */
   public boolean isCommandSupportsAdvancedOptions(String command)
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION,
             "<Perforce Provider Type>: isCommandSupportsAdvancedOptions called");
      }
      return CMD_CHECKOUT_FILES.equals(command) || CMD_CHECKIN_FILES.equals(command) || CMD_GET_FILES.equals(command) ||
          CMD_ADD_FILES.equals(command);
   }

   /**
    * Returns project name valid for the Perforce provider which corresponds to working
    * directory <CODE>workingDir</CODE>, given that project <CODE>rootProject</CODE>
    * corresponds to root working directory <CODE>rootWorkingDir</CODE>.
    * @param rootProject project valid for this VcsProviderType corresponding to root working directory
    * @param rootWorkingDir root working directory
    * @param workingDir working directory to calculate valid project name for
    * @return <code>true</code> if project name <CODE>project</CODE> is valid
    */
   public String getValidProjectName(String rootProject, File rootWorkingDir, File workingDir) throws VcsProviderException
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: getValidProjectName called");
      }
      return "<Perforce Project Root><n>";
   }

   /**
    * Returns <code>true</code> if <CODE>project</CODE> is valid for the Perforce provider,
    * that is - not null and not "" and its a perforce client name that has the same host name as the local host.
    * @return <code>true</code> if project name <CODE>project</CODE> is valid
    */
   public boolean isProjectNameValid(String project)
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: isProjectNameValid called");
      }
      if (null != project && project.startsWith("<Perforce Project Root>"))
      {
         return true;
      }
      Client c = Client.getClient(getEnv(), project);
      if (c == null)
         return false;
      c.sync();
      if (myHost != null && myHost.equals(c.getHost()))
      {
         return true;
      }
      else
      {
         return false;
      }
   }

   /**
    * Returns <code>true</code> if the Perforce provider instance is registerd with the Perforce provider type.
    * @return <code>true</code> if <CODE>provider</CODE> instance is valid
    */
   public boolean isProviderValid(VcsProvider provider)
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: isProviderValid called");
      }
      return VcsProviderHelper.getInstance().isProviderRegistered(provider);
   }

   /**
    * Invalidates specified <CODE>provider</CODE>. An invalidated provider
    * is not functional, its behaviour is not determined. The invalidated
    * instance of <code>VcsProvider</code> must free all allocated resources and be prepared to garbage collection.
    * @param provider provider instance to invalidated
    */
   public void invalidateProvider(VcsProvider provider)
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: invalidateProvider called");
      }
      VcsProviderHelper.getInstance().unRegisterProvider(provider);
   }

   /** Invalidates all providers registered uder the ID <code>getID()</code>. */
   public void invalidateAllProviders()
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: invalidateAllProviders called");
      }
      VcsProviderHelper.getInstance().unRegisterAllProviders(getID());
   }

   /**
    * Returns all the commands supported by the Perforce provider.
    * @return <code>String</code> array of supported commands
    */
   public String[] supportedCommands()
   {
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: supportedCommands called");
      }
      return new String[]
      {
         CMD_SELECT_PROJECT, CMD_GET_FILES_INFO, CMD_CHECKOUT_FILES, CMD_CHECKIN_FILES, CMD_UPDATE_FILES, CMD_SHOW_FILE_DIFF,
             CMD_SHOW_FILES_HISTORY, CMD_GET_FILES, CMD_SHOW_FILE_PROPERTIES, CMD_UNCHECKOUT_FILES, CMD_REMOVE_FILES,
             CMD_RENAME_FILE, CMD_LOCK_FILES, CMD_UNLOCK_FILES, CMD_GET_FILES_STATUSES, CMD_ADD_FILES
      };
   }

   public void propertyChange(PropertyChangeEvent evt)
   {
      String propName = evt.getPropertyName();
      if (propName != null && propName.startsWith("vcs.perforce"))
      {
         propName = propName.substring(13);
         String value = (String)evt.getNewValue();
         if ("debug".equals(propName) && null != value && value.equals("true"))
         {
            debugOn = true;
         }
         else
         {
            debugOn = false;
         }
         myProperties.setProperty("p4." + propName, value);
         p4Env = new Env(getProperties());
         Debug.setProperties(getProperties());
         String logFileName = getProperties().getProperty("p4.logfile");
         EventLog eventLog = null;
         if (null != logFileName && !"".equals(logFileName))
         {
            eventLog = new EventLog("Together Perforce Log", logFileName);
         } else
         {
            eventLog = new EventLog("Together Perforce Log");
         }
         Debug.setEventLog(eventLog);         
         VcsProviderRegistryAccess.getRegistry().invalidateProvider(getID());
      }
   }

   /**
    * Returns the <code>Env</code> object instance.
    * @return <code>Env</code>
    */
   private Env getEnv()
   {
      return p4Env;
   }

   /**
    * Returns the property values loaded from the current project configuration.
    * @return <code>Properties</code> the loaded Properties instance
    */
   private Properties loadProperties()
   {
      IdeConfig myConfig = IdeAccess.getConfigManager().getConfig(IdeConfigLevel.PROJECT);
      Properties props = new Properties(System.getProperties());
      String dbg = myConfig.getProperty("vcs.perforce.debug", "false");
      if (dbg.equals("true"))
      {
         debugOn = true;
      }
      else
      {
         debugOn = false;
      }
      if (debugOn && messagepage != null)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider Type>: loadProperties called");
      }
      for (int i = 0; i < propNames.length; i++)
      {
         String propName = propNames[i];
         props.setProperty("p4." + propName, myConfig.getProperty("vcs.perforce." + propName, ""));
      }
      return props;
   }

   static Properties getProperties()
   {
      return myProperties;
   }

   private boolean isInitialised = false;
   private IdeMessagePage messagepage;
   private boolean debugOn = true;
   private static Properties myProperties = null;
   private Env p4Env;
   private String myHost;

   private String[] propNames = new String[]
   {
      "client", "user", "port", "password", "executable", "sysdrive", "sysroot", "threshold", "debug_level", "log_level", "debug", "diff"
   };
}
